#include <IRstD.h>

IRrecvstd irrecvstd(10); // пин приёмника (можно любой кроме пина передатчика)
                         // пин передатчика для Duemilanove, Diecimila, LilyPad, Mini, Fio, Nano - 9, для Мега - 5

void  setup ()
{
  Serial.begin(57600);   
  irrecvstd.enableIRInstd();
}


void  ircode(decode_resultsstd *results)
{
  if (results->decode_typestd == PANASONICstd) {
    Serial.print(results->addressstd, HEX);
    Serial.print(":");
  }

  Serial.print(results->valuestd, HEX);
}


void  encoding (decode_resultsstd *results)
{
  switch (results->decode_typestd) {
    default:
    case UNKNOWNstd:      Serial.print("UNKNOWN");       break ;
    case NECstd:          Serial.print("NEC");           break ;
    case SONYstd:         Serial.print("SONY");          break ;
    case RC5std:          Serial.print("RC5");           break ;
    case RC6std:          Serial.print("RC6");           break ;
    case DISHstd:         Serial.print("DISH");          break ;
    case SHARPstd:        Serial.print("SHARP");         break ;
    case JVCstd:          Serial.print("JVC");           break ;
    case SANYOstd:        Serial.print("SANYO");         break ;
    case MITSUBISHIstd:   Serial.print("MITSUBISHI");    break ;
    case SAMSUNGstd:      Serial.print("SAMSUNG");       break ;
    case LGstd:           Serial.print("LG");            break ;
    case WHYNTERstd:      Serial.print("WHYNTER");       break ;
    case AIWA_RC_T501std: Serial.print("AIWA_RC_T501");  break ;
    case PANASONICstd:    Serial.print("PANASONIC");     break ;
    case DENONstd:        Serial.print("Denon");         break ;
  }
}


void  dumpInfo (decode_resultsstd *results)
{
  if (results->overflowstd) {
    Serial.println("IR code too long. Edit IRremoteInt.h and increase RAWLEN");
    return;
  }

 
  Serial.print("Encoding  : ");
  encoding(results);
  Serial.println("");
  Serial.print("Code      : ");
  ircode(results);
  Serial.print(" (");
  Serial.print(results->bitsstd, DEC);
  Serial.println(" bits)");
}


void  dumpRaw (decode_resultsstd *results)
{
 
  Serial.print("Timing[");
  Serial.print(results->rawlenstd-1, DEC);
  Serial.println("]: ");

  for (int i = 1;  i < results->rawlenstd;  i++) {
    unsigned long  x = results->rawbufstd[i] * USECPERTICK;
    if (!(i & 1)) {  
      Serial.print("-");
      if (x < 1000)  Serial.print(" ") ;
      if (x < 100)   Serial.print(" ") ;
      Serial.print(x, DEC);
    } else { 
      Serial.print("     ");
      Serial.print("+");
      if (x < 1000)  Serial.print(" ") ;
      if (x < 100)   Serial.print(" ") ;
      Serial.print(x, DEC);
      if (i < results->rawlenstd-1) Serial.print(", "); //',' not needed for last one
    }
    if (!(i % 8))  Serial.println("");
  }
  Serial.println("");                    
}


void  dumpCode (decode_resultsstd *results)
{
  Serial.print("unsigned int  ");         
  Serial.print("rawData[");               
  Serial.print(results->rawlenstd - 1, DEC);  
  Serial.print("] = {");                 

  for (int i = 1;  i < results->rawlenstd;  i++) {
    Serial.print(results->rawbufstd[i] * USECPERTICK, DEC);
    if ( i < results->rawlenstd-1 ) Serial.print(","); // ',' not needed on last one
    if (!(i & 1))  Serial.print(" ");
  }

  Serial.print("};"); 

  Serial.print("  // ");
  encoding(results);
  Serial.print(" ");
  ircode(results);

  Serial.println("");

  if (results->decode_typestd != UNKNOWNstd) {

    if (results->decode_typestd == PANASONICstd) {
      Serial.print("unsigned int  addr = 0x");
      Serial.print(results->addressstd, HEX);
      Serial.println(";");
    }

    Serial.print("unsigned int  data = 0x");
    Serial.print(results->valuestd, HEX);
    Serial.println(";");
  }
}


void loop()
{
  decode_resultsstd  results;        

  if (irrecvstd.decodestd(&results)) {  
    dumpInfo(&results);           
    dumpRaw(&results);           
    dumpCode(&results);      
    Serial.println("");         
    irrecvstd.resumestd();            
  }
}


