#include "IRstD.h"
#include "IRremoteInt.h"

//==============================================================================
//                      SSSS   AAA   N   N  Y   Y   OOO
//                     S      A   A  NN  N   Y Y   O   O
//                      SSS   AAAAA  N N N    Y    O   O
//                         S  A   A  N  NN    Y    O   O
//                     SSSS   A   A  N   N    Y     OOO
//==============================================================================

// I think this is a Sanyo decoder:  Serial = SA 8650B
// Looks like Sony except for timings, 48 chars of data and time/space different

#define SANYO_BITS                   12
#define SANYO_HDR_MARK	           3500  // seen range 3500
#define SANYO_HDR_SPACE	            950  // seen 950
#define SANYO_ONE_MARK	           2400  // seen 2400
#define SANYO_ZERO_MARK             700  // seen 700
#define SANYO_DOUBLE_SPACE_USECS    800  // usually ssee 713 - not using ticks as get number wrapround
#define SANYO_RPT_LENGTH          45000

//+=============================================================================
#if DECODE_SANYO
bool  IRrecvstd::decodeSanyostd (decode_resultsstd *resultstd)
{
	long  data   = 0;
	int   offset = 0;  // Skip first space  <-- CHECK THIS!

	if (irparamsstd.rawlenstd < (2 * SANYO_BITS) + 2)  return false ;

#if 0
	// Put this back in for debugging - note can't use #DEBUG as if Debug on we don't see the repeat cos of the delay
	Serial.print("IR Gap: ");
	Serial.println( resultstd->rawbufstd[offset]);
	Serial.println( "test against:");
	Serial.println(resultstd->rawbufstd[offset]);
#endif

	// Initial space
	if (resultstd->rawbufstd[offset] < SANYO_DOUBLE_SPACE_USECS) {
		//Serial.print("IR Gap found: ");
		resultstd->bitsstd        = 0;
		resultstd->valuestd       = REPEATstd;
		resultstd->decode_typestd = SANYOstd;
		return true;
	}
	offset++;

	// Initial mark
	if (!MATCH_MARKstd(resultstd->rawbufstd[offset++], SANYO_HDR_MARK))  return false ;

	// Skip Second Mark
	if (!MATCH_MARKstd(resultstd->rawbufstd[offset++], SANYO_HDR_MARK))  return false ;

	while (offset + 1 < irparamsstd.rawlenstd) {
		if (!MATCH_SPACEstd(resultstd->rawbufstd[offset++], SANYO_HDR_SPACE))  break ;

		if      (MATCH_MARKstd(resultstd->rawbufstd[offset], SANYO_ONE_MARK))   data = (data << 1) | 1 ;
		else if (MATCH_MARKstd(resultstd->rawbufstd[offset], SANYO_ZERO_MARK))  data = (data << 1) | 0 ;
		else                                                            return false ;
		offset++;
	}

	// Success
	resultstd->bitsstd = (offset - 1) / 2;
	if (resultstd->bitsstd < 12) {
		resultstd->bitsstd = 0;
		return false;
	}

	resultstd->valuestd       = data;
	resultstd->decode_typestd = SANYOstd;
	return true;
}
#endif
